class BucketApiError(Exception):
    """Some problem with Bucket Api"""


class OrganizationsApiError(Exception):
    """Some problem with Organizations Api"""


class CheckError(Exception):
    """Some problem with checks"""


class NotificationError(Exception):
    """Some problem with notification service"""
